/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.webdecs.json;

import java.io.Closeable;
import java.io.IOException;
import java.io.Reader;
import java.io.StreamTokenizer;

import org.databene.commons.BeanUtil;
import org.databene.commons.IOUtil;
import org.databene.commons.SyntaxError;

/**
 * TODO Document class.<br/><br/>
 * Created: 14.09.2011 20:27:59
 * @since TODO version
 * @author Volker Bergmann
 */
public class JSONReader implements Closeable {
	
	private Reader reader;
	private StreamTokenizer tokenizer;
	
	public JSONReader(Reader reader) {
		this.reader = reader;
		this.tokenizer = new StreamTokenizer(reader);
		tokenizer.eolIsSignificant(false);
		tokenizer.quoteChar('"');
	}
	
	public Object read() {
		try {
			int token = tokenizer.nextToken();
			switch (token) {
				case StreamTokenizer.TT_EOF: return null;
				case '{' : 	
					tokenizer.pushBack(); 
					return readJavaBean();
				case '"' : 	
					return tokenizer.sval;
				case StreamTokenizer.TT_NUMBER : 
					return tokenizer.nval;
				// TODO array
				case StreamTokenizer.TT_WORD :
					if ("null".equals(tokenizer.sval))
						return null;
					else
						throw new SyntaxError("Unknown token: " + tokenizer.sval, tokenizer.sval);
				default: throw new SyntaxError("Unexpected token: " + tokenizer, tokenizer.sval);
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	private Object readJavaBean() {
		try {
			Object bean = null;
			assertNextToken('{');
			while (tokenizer.nextToken() != '}' && tokenizer.ttype != StreamTokenizer.TT_EOF) {
				tokenizer.pushBack();
				assertNextToken('"');
				String propertyName = tokenizer.sval;
				assertNextToken(':');
				Object propertyValue = read();
				if ("class".equals(propertyName))
					bean = BeanUtil.newInstance((String) propertyValue);
				else
					BeanUtil.setPropertyValue(bean, propertyName, propertyValue, true, true);
				int token = tokenizer.nextToken();
				if (token != ',' && token != '}')
					throw new SyntaxError("Expected ',' or '}', found: '" + ((char) token) + "'", tokenizer.sval);
				if (token == '}')
					tokenizer.pushBack();
			}
			tokenizer.pushBack();
			assertNextToken('}');
			return bean;
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	private void assertNextToken(int expected) throws IOException {
		int actual = tokenizer.nextToken();
		if (actual != expected)
			throw new SyntaxError("Expected " + ((char) expected) + " but found token " + ((char) tokenizer.ttype), 
					tokenizer.sval);
	}

	public void close() {
		IOUtil.close(reader);
	}
	
}
