/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.webdecs.json;

import java.beans.PropertyDescriptor;
import java.io.Closeable;
import java.io.PrintWriter;

import org.databene.commons.BeanUtil;

/**
 * TODO Document class.<br/><br/>
 * Created: 14.09.2011 18:02:53
 * @since TODO version
 * @author Volker Bergmann
 */
public class JSONWriter implements Closeable {
	
	private PrintWriter out;
	private boolean storeClass;
	private StringBuilder indent;

	public JSONWriter(PrintWriter out, boolean storeClass) {
		this.out = out;
		this.storeClass = storeClass;
		this.indent = new StringBuilder();
	}
	
	public void print(Object object) {
		if (object == null) {
			out.print("null");
		} else if (object instanceof String || object instanceof Character) {
			printQuoted(object);
		} else {
			// TODO array
			Class<?> type = object.getClass();
			if (BeanUtil.isSimpleType(type.getName()))
				out.print(object);
			else
				printJavaBean(object);
		}
	}

	private void printJavaBean(Object bean) {
		try {
			out.print(indent);
			out.println('{');
			indent.append('\t');
			boolean firstProp = true;
			if (storeClass) {
				printNameAndValue("class", bean.getClass().getName());
				firstProp = false;
			}
			PropertyDescriptor[] descriptors = BeanUtil.getPropertyDescriptors(bean.getClass());
			for (PropertyDescriptor descriptor : descriptors) {
				if (!"class".equals(descriptor.getName())) {
					if (!firstProp)
						out.println(',');
					printNameAndValue(descriptor.getName(), descriptor.getReadMethod().invoke(bean));
					firstProp = false;
				}
			}
			indent.delete(indent.length() - 1, indent.length());
			out.println();
			out.print(indent);
			out.println('}');
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	private void printNameAndValue(String name, Object value) {
		out.print(indent);
		printQuoted(name);
		out.print(": ");
		print(value);
	}

	protected void printQuoted(Object object) {
		out.print('"');
		out.print(object);
		out.print('"');
	}

	public void close() {
		out.close();
	}
	
}
